-- 
-- ESTIMATE FUNCTIONS
--

--
-- ADDED SPELLPOWER
--
-- take a spell list and tells you how much you'd gain with added spell power
function ShadowSeer_EstimateAddedDamageWithSpellPower(dlist, sp)
	if (UnitClass("player") ~= "Warlock") then
		return 0
	end

	local bonusd = 0
	for j = 2, table.getn(dlist) do
		bonusd = bonusd + ShadowSeer_EstimateBonusDamage(dlist[j], sp)
	end
	return floor(bonusd - dlist[1].totaldamage + 0.5)
end


-- takes a spell list and a spell, and +damage bonus. Tells you how much damage would be added
function ShadowSeer_EstimateBonusDamage(slist, damagebonus)
	local real_average_tic_damage = slist.totaldamage / slist.ticcount
	local estimated_average_tic_damage = ShadowSeer_EstimateAverageTic(slist.spellname)

	if (estimated_average_tic_damage == 0) then
		return 0 -- unknown spell (like mark of kaz'rogal)
	end
	local damageratio = real_average_tic_damage / estimated_average_tic_damage

	local estimated_average_tic_damage_with_modded_power = ShadowSeer_EstimateAverageTic(slist.spellname, damagebonus)
	local estimated_real_tic_damage_with_modded_power = damageratio * estimated_average_tic_damage_with_modded_power

--  ShadowSeer_ShowMessage(slist.spellname)
--	ShadowSeer_ShowMessage("real_average_tic_damage= " .. real_average_tic_damage .. " estimated_average_tic_damage= " .. estimated_average_tic_damage)
--	ShadowSeer_ShowMessage("damageratio = " .. damageratio .. " estimated_average_tic_damage_with_modded_power= " .. estimated_average_tic_damage_with_modded_power)
--	ShadowSeer_ShowMessage("estimated_real_tic_damage_with_modded_power = " .. estimated_real_tic_damage_with_modded_power)
--	ShadowSeer_ShowMessage("old damage = " .. slist.totaldamage .. " new damage =  " .. floor(estimated_real_tic_damage_with_modded_power * slist.ticcount + 0.5))
	
	return floor(estimated_real_tic_damage_with_modded_power * slist.ticcount + 0.5)
end

-- estimates how hard a spell tics, based on your average spell bonus. takes current bonus if you didn't cast spells yet
function ShadowSeer_EstimateAverageTic(spell, bonus)
	local bonusdamage = 0
	if (bonus) then
		bonusdamage = bonus
	end
	
	local school = ShadowSeer_GetSchoolForSpell(spell)
	local sl = ShadowSeer_GetActivePlayer().spellinfo
	local spower = 0
	if (school == "Shadow") then
		if (sl.shadowcount > 0) then
			spower = sl.shadowpower / sl.shadowcount
		else
			spower = GetSpellBonusDamage(6)
		end
	elseif (school == "Fire") then
		if (sl.firecount > 0) then
			spower = sl.firepower / sl.firecount
		else
			spower = GetSpellBonusDamage(3)
		end
	end
	
	return ShadowSeer_EstimateAverageTicWithSpellpower(spell, spower + bonusdamage)
end

-- takes a spell list and tells you how much you'd gain with spell crit. Takes a spell field and the gained amount of spell crit
function ShadowSeer_EstimateAddedDamageWithCritRating(dlist, cr)
	if (UnitClass("player") ~= "Warlock") then
		return 0
	end

	local bonusd = 0
	for j = 2, table.getn(dlist) do
		bonusd = bonusd + ShadowSeer_EstimateSpellAddedDamageWithCritRating(dlist[j], cr)
	end
	return floor(bonusd + 0.5)
end

-- takes a spell list and a crit bonus and estimated how much you'd gain with crit
function ShadowSeer_EstimateSpellAddedDamageWithCritRating(slist, cr)
	if (ShadowSeer_SpellCanCrit(slist.spellname)) then
		local avg_non_crit = 0
		local avg_crit = 0
		local crit_multiplier = 1.5 + 0.5 * ShadowSeer_TalentsIn("Ruin")
		if (slist.spellname == "Seed of Corruption") then
			crit_multiplier = 1.5
		end
		avg_crit = slist.totalcritdamage / max(1, slist.critcount)
		
		if (slist.ticcount == slist.critcount) then -- all of the spells have crit. for fun, determine a noncrit hit
			avg_non_crit = avg_crit / crit_multiplier
		else
			avg_non_crit = (slist.totaldamage - slist.totalcritdamage) / (slist.ticcount - slist.critcount)
		end

		if (slist.critcount == 0) then -- player has not crit. estimate at 150 or 200% of normal hit, depending on ruin
			avg_crit = avg_non_crit * crit_multiplier
		end
		
		local pl = ShadowSeer_GetActiveList()[1]
		local average_isb_contribution = pl.ISBcontribution / 6 / max(1, pl.SBcrits)
		-- add isb contrib
		if (slist.spellname == "Shadow Bolt" and ShadowSeer_TalentsIn("Improved Shadow Bolt") == 5) then 
			avg_crit = avg_crit + average_isb_contribution
		end
		
		local current_crit_ratio = slist.critcount / slist.ticcount
		local new_crit_ratio = min(current_crit_ratio + cr / 22.1 / 100, 1)
		local estimated_new_amount_of_crits = slist.ticcount * new_crit_ratio
		local estimated_new_amount_of_noncrits = slist.ticcount - estimated_new_amount_of_crits
		
		local new_total_damage = floor(avg_non_crit * (slist.ticcount - estimated_new_amount_of_crits) + avg_crit * estimated_new_amount_of_crits + 0.5)

		local old_total_damage = floor(avg_non_crit * (slist.ticcount - slist.critcount) + avg_crit * slist.critcount + 0.5) -- this includes ISB bonus
		
		--ShadowSeer_ShowMessage(slist.spellname .. ": " .. slist.critcount .. "/" .. slist.ticcount .. " for " .. slist.totalcritdamage .. "/".. slist.totaldamage .. " damage.")
		--ShadowSeer_ShowMessage(slist.spellname .. ": avg noncrit = " .. floor(avg_non_crit) .. " avg crit = " .. floor(avg_crit) .. " avg_isb_contrib = " .. floor (average_isb_contribution))
		--ShadowSeer_ShowMessage(slist.spellname .. ": current crit% = " .. current_crit_ratio .. " new ratio = " .. new_crit_ratio .. " est crit count = " .. floor(estimated_new_amount_of_crits*100)/100) 
		--ShadowSeer_ShowMessage(slist.spellname .. ": new total damage = " .. new_total_damage .. " gain = " .. new_total_damage - slist.totaldamage)

		return new_total_damage - old_total_damage
	else -- can't crit - no gains
		return 0
	end
end

-- estimates how hard a spell tics given a +dmg bonus (non crits)
function ShadowSeer_EstimateAverageTicWithSpellpower(spell, sp)
	local multiplier = 1 + 0.02 * ShadowSeer_TalentsIn("Shadow Mastery")
	if (ShadowSeer_GetSchoolForSpell(spell) == "Fire") then
		multiplier = 1 + 0.02 * ShadowSeer_TalentsIn("Emberstorm")
	end
	
	if (spell == "Curse of Doom") then
		mulitplier = 1 -- not affected for some reason
	end

	return ShadowSeer_EstimateAverageTicWithSpellpowerIgnoringMastery(spell, sp) * multiplier
end

function ShadowSeer_EstimateAverageTicWithSpellpowerIgnoringMastery(spell, sp)
	if (spell == "Corruption [DOT]") then -- checks out. 0.94 is not consistent with others, it's an exception
		return (900 + (0.94 + .12 * ShadowSeer_TalentsIn("Empowered Corruption")) * sp) * (1 + 0.01 * ShadowSeer_TalentsIn("Contagion")) / 6
	elseif (spell == "Shadow Bolt") then
		return (544+607)/2 + (sp * (3 / 3.5 + 0.04 * ShadowSeer_TalentsIn("Shadow and Flame")))
	elseif (spell == "Curse of Doom [DOT]") then -- yup, exception
		return 4200 + 2 * sp
	elseif (spell == "Curse of Agony [DOT]") then -- 1.2 is an exception too
		return (1356 + 1.2 * sp) * (1 + 0.05 * ShadowSeer_TalentsIn("Improved Curse of Agony")) * (1 + 0.01 * ShadowSeer_TalentsIn("Contagion")) / 12
	elseif (spell == "Shadowburn") then
		return (597 + 665) / 2 + (1.5 / 3.5) * sp
	elseif (spell == "Drain Life [DOT]") then -- 5s / 3.5  divided by 2 due to bonus effect (gain life)
		return (540 + sp * 5 / 2 / 3.5) / 5 -- ignoring the talent that adds damage per debuff. 
	elseif (spell == "Drain Soul [DOT]") then -- we're assuming rank 1
		return (55 + sp * 1.06) / 4  
	elseif (spell == "Death Coil") then -- 1.5s / 3.5  divided by 2 due to bonus effect (gain life)
		return (525 + sp * 1.5 / 2 / 3.5) 
	elseif (spell == "Seed of Corruption [DOT]") then -- another exception, gets 150% from damage
		return (1044 + sp * 1.5) * (1 + 0.01 * ShadowSeer_TalentsIn("Contagion")) / 6
--	elseif (spell == "Seed of Corruption") then -- direct impact gets 16% it seems
	elseif (spell == "Seed [DETONATION]") then -- direct impact gets 16% it seems
		return (1200 + sp / 6) * (1 + 0.01 * ShadowSeer_TalentsIn("Contagion"))
	elseif (spell == "Siphon Life [DOT]") then -- NOT an exception 30/15 = 200%, halved cause of life gain
		return (630 + sp) / 10
	elseif (spell == "Unstable Affliction [DOT]") then -- NOT an exception! 18/15 = 120% damage from items
		return (1044 + sp * 1.2) / 6
	elseif (spell == "Shadowfury") then -- seems to check out, damage bonus halved due to bonus effect (stun)
		return (612+728)/2 + (sp * 1.5/3.5 / 2)
	
	-- fire spells
	elseif (spell == "Immolate") then
		return 331.5 + sp * 0.20
	elseif (spell == "Immolate [DOT]") then
		return (615 + sp * 0.65) / 5
	elseif (spell == "Hellfire" or spell == "Hellfire Effect") then 
		return (307.5 * 15 + sp * 7.5 / 3.5) / 15
	elseif (spell == "Incinerate") then
		return (452+525)/2 + (113+132)/2 + sp * (2.5 / 3.5 + 0.2 * ShadowSeer_TalentsIn("Shadow and Flame"))
	elseif (spell == "Rain of Fire") then
		return (1212 + sp * 1.148) / 4
	elseif (spell == "Searing Pain") then 
		return (270+320)/2 + sp * 1.5 / 3.5
	elseif (spell == "Soul Fire") then
		return 1153 + sp * 1.15
	elseif (spell == "Conflagrate") then
		return 0
	else 
		return 0
	end
end

-- calculates gains by adding +hit. 
-- resulttable = { spellname, resultlist }
-- resultlist = (( level , resistcount, castcount, critcount, damagetotal ))
function ShadowSeer_EstimateTimeAndDamageGainedWithHitRating(hr)
	if (UnitClass("player") ~= "Warlock") then
		return 0,0
	end
	
	local pr = ShadowSeer_GetActivePlayer()
	local rt = pr.resulttable

	-- calculate lost cast time 
	local gain = 0
	for i = 1, table.getn(rt) do
		for j = 1, table.getn(rt[i].resultlist) do 
			gain = gain + ShadowSeer_EstimateTimeGained(rt[i].spellname, rt[i].resultlist[j], hr)
		end
	end

	-- convert into shadow bolt damage
	local dl = pr.damagelist
	local bolt_damage = 0
	local bolt_crit_ratio = 0
	
	for i = 1, table.getn(dl) do
		if (dl[i].spellname == "Shadow Bolt") then
			bolt_damage = dl[i].totaldamage / dl[i].ticcount
			bolt_crit_ratio = dl[i].critcount / dl[i].ticcount
		end
	end
	
	local pl = ShadowSeer_GetActiveList()[1]
	local average_isb_contribution = pl.ISBcontribution / 6 / max(1, pl.SBcrits)
	if (ShadowSeer_TalentsIn("Improved Shadow Bolt") < 5) then
		average_isb_contribution = 0
	end
	
	-- average bolt does this much
	local hchance = min(ShadowSeer_GetHitChanceVs("BOSS") + GetCombatRatingBonus(CR_HIT_SPELL)/100 + hr / 12.6 / 100, .99)
	local avg_sb_damage = (bolt_damage + average_isb_contribution * bolt_crit_ratio) * hchance
	local bdamage = gain / ShadowSeer_GetCastingTime("Shadow Bolt") * avg_sb_damage
	
--	ShadowSeer_ShowMessage("DEBUG: total gained " .. gain .. "s, avg_sb_damage = " .. avg_sb_damage .. " sbct = " .. ShadowSeer_GetCastingTime("Shadow Bolt"))
	
	return floor(gain / ShadowSeer_GetCastingTime("Shadow Bolt")*10)/10, floor(gain / ShadowSeer_GetCastingTime("Shadow Bolt") * avg_sb_damage)
end

function ShadowSeer_HitRatingNeededTillCap()
	local resistchance = max(1 - ShadowSeer_GetHitChanceVs("BOSS") - GetCombatRatingBonus(CR_HIT_SPELL)/100, 0.01)
	local hr = (resistchance - 0.01) * 100 * 12.6
	return hr
end

-- playerlist
-- ( "name" = name, "class" = class, "damagelist" = damagelist, "resulttable" = resulttable, "ISBcontribution" = ISBcontribution, "SBcrits" = sbcrits, ")
-- first entry is "raidwide"

-- Damagelist = (( spellname, critcount, ticcount, fullresistcount, totaldamage, maxdamage, ampeddamage,totalcritdamage )) -- ampeddamage lists the % of it that was added by ISB
-- first entry is "total"


-- estimate the difference in casting time lost with a new hit rating
function ShadowSeer_EstimateTimeGained(spell, rl, hr)
	if (spell == "Shadowfury" or spell == "Seed of Corruption [DOT]" or spell == "Seed [DETONATION]" or spell == "Hellfire Effect" or spell == "Hellfire" or spell == "Rain of Fire") then
		return 0 -- assume aoe doesn't gain from +hit since it's mostly against target that are hit capped 
	end
	
	if (spell == "Soulshatter") then
		return 0
	end

	local old_hit_bonus = GetCombatRatingBonus(CR_HIT_SPELL)/100
	if (ShadowSeer_IsAfflictionSpell(spell)) then
		old_hit_bonus = old_hit_bonus + 0.02 * ShadowSeer_TalentsIn("Suppression")
	end

	-- sanity check. if our theoretical resist rate is capped, no gains
	local hit_chance = ShadowSeer_GetHitChanceVs(rl.level) + old_hit_bonus
	if (hit_chance > .99) then
		return 0
	end

	-- if our real resist rate was capped, no gains
	local real_hit_count = rl.castcount - rl.resistcount
	local real_hit_rate = real_hit_count/rl.castcount
	if (real_hit_rate >= .99) then
		return 0
	end
	
	-- cap new resist rate at 99%
	local gain = hr / 12.6 / 100
	local new_hit_rate = min(real_hit_rate + gain, 0.99)
	local new_hit_count = new_hit_rate*rl.castcount

--	ShadowSeer_ShowMessage("DEBUG: " .. spell .. " vs " .. rl.level .. " hit chance=" .. hit_chance .. " real hc= " .. real_hit_rate .. " new_hit_rate= " .. new_hit_rate .. " extra hits " .. new_hit_count - real_hit_count .. " cast time= " .. ShadowSeer_GetCastingTime(spell))
	
	return (new_hit_count - real_hit_count) * ShadowSeer_GetCastingTime(spell)
end

function ShadowSeer_GetHitChanceVs(level)
	if (level == "BOSS" or level == 73) then return .83
	elseif (level == 72) then return .94
	elseif (level == 71) then return .95
	elseif (level == "UNKNOWN" or not level or level == 70) then return .96
	elseif (level == 69) then return .97
	elseif (level == 68) then return .98
	else return .99
	end
end

function ShadowSeer_GetCastingTime(spellname, bonushasterating)
	-- filter dot out
	local spell = string.gsub(spellname, " %[DOT%]", "")
	
	if (not bonushasterating) then
		bonushasterating = 0
	end
	local ctime = ShadowSeer_GetCastingTimeIgnoringHaste(spell)
	local hasterating = GetCombatRating(CR_HASTE_SPELL) + bonushasterating
	local newspeed = 1 + hasterating / 15.77 / 100
	return max(1.5, ctime / newspeed)
end

function ShadowSeer_GetCastingTimeIgnoringHaste(spell)
	if (spell == "Corruption") then
		return max(2 - 0.4 * ShadowSeer_TalentsIn("Improved Corruption"), 1.5)
	elseif (spell == "Shadow Bolt") then
		return 3 - 0.1 * ShadowSeer_TalentsIn("Bane")
	elseif (spell == "Seed of Corruption" or spell == "Seed [DETONATION]") then
		return 2
	elseif (spell == "Soul Fire") then 
		return 6 - 0.4 * ShadowSeer_TalentsIn("Bane")
	elseif (spell == "Incinerate") then 
		return 2.5
	elseif (spell == "Immolate") then 
		return 2 - 0.1 * ShadowSeer_TalentsIn("Bane")
	else
		return 1.5
	end
end
-- for fire, add these
--	elseif (spellname == "Incinerate") then return 2.5
--	elseif (spellname == "Soul Fire") then return 6 - 0.4 * ShadowSeer_TalentsIn("Bane")
--	elseif (spellname == "Immolate") then return 2 - 0.1 * ShadowSeer_TalentsIn("Bane")

function ShadowSeer_IsAfflictionSpell(spellname)
-- channeled
	if (spellname == "Drain Life [DOT]") then return true
	elseif (spellname == "Drain Soul [DOT]") then return true
	elseif (spellname == "Drain Mana [DOT]") then return true

-- curses
	elseif (spellname == "Curse of Agony [DOT]") then return true
	elseif (spellname == "Curse of Shadow") then return true
	elseif (spellname == "Curse of the Elements") then return true
	elseif (spellname == "Curse of Doom [DOT]") then return true
	elseif (spellname == "Curse of Exhaustion") then return true
	elseif (spellname == "Curse of Recklessness") then return true
	elseif (spellname == "Curse of Tongues") then return true
	elseif (spellname == "Curse of Weakness") then return true

-- dots
	elseif (spellname == "Seed of Corruption [DOT]") then return true
	elseif (spellname == "Corruption [DOT]") then return true
	elseif (spellname == "Unstable Affliction [DOT]") then return true
	elseif (spellname == "Siphon Life [DOT]") then return true

	else return false
	end
end	

function ShadowSeer_EstimateTimeAndDamageGainedWithHasteRating(hr)
	if (UnitClass("player") ~= "Warlock") then
		return 0,0
	end

	local pr = ShadowSeer_GetActivePlayer()
	local rt = pr.resulttable
	local dl = pr.damagelist

	-- calc average shadow bolt damage and dl tics
	local bolt_damage = 0
	local bolt_crit_ratio = 0
	local drain_life_tics = 0
	
	for i = 1, table.getn(dl) do
		if (dl[i].spellname == "Shadow Bolt") then
			bolt_damage = dl[i].totaldamage / dl[i].ticcount
			bolt_crit_ratio = dl[i].critcount / dl[i].ticcount
		-- keep track of time spent draining life
		elseif (dl[i].spellname == "Drain Life [DOT]") then
			drain_life_tics = dl[i].ticcount
		end
	end
	
	local hchance = min(ShadowSeer_GetHitChanceVs("BOSS") + GetCombatRatingBonus(CR_HIT_SPELL)/100, .99)
	local pl = ShadowSeer_GetActiveList()[1]
	local average_isb_contribution = pl.ISBcontribution / 6 / max(1, pl.SBcrits)
	if (ShadowSeer_TalentsIn("Improved Shadow Bolt") < 5) then
		average_isb_contribution = 0
	end
	local avg_sb_damage = (bolt_damage + average_isb_contribution * bolt_crit_ratio) * hchance
	
	
	-- calculate total cast time
	local totalcasttime = 0
	local totalnewtime = 0
	for i = 1, table.getn(rt) do
		if (rt[i].spellname == "Drain Life [DOT]") then -- ignoring drain soul & ROF. can't reliably determine gains.
			totalcasttime = totalcasttime + drain_life_tics / (1 + GetCombatRating(CR_HASTE_SPELL)/ 15.77 / 100)
			totalnewtime = totalnewtime + drain_life_tics / (1 + (GetCombatRating(CR_HASTE_SPELL) + hr) / 15.77 / 100)
		elseif (rt[i].spellname == "Hellfire") then 
			totalcasttime = totalcasttime + rt[i].resultlist[1].selfticcount / (1 + GetCombatRating(CR_HASTE_SPELL)/ 15.77 / 100)
			totalnewtime = totalnewtime + rt[i].resultlist[1].selfticcount / (1 + (GetCombatRating(CR_HASTE_SPELL) + hr) / 15.77 / 100)
		else
			local totalcasts = 0
			for j = 1, table.getn(rt[i].resultlist) do 
				totalcasts = totalcasts + rt[i].resultlist[j].castcount
			end
		
			totalcasttime = totalcasttime + ShadowSeer_GetCastingTime(rt[i].spellname) * totalcasts
			totalnewtime = totalnewtime + ShadowSeer_GetCastingTime(rt[i].spellname, hr) * totalcasts
		end
	end
	-- calculate gained time
	local gained_time = totalcasttime - totalnewtime

	-- calculate mana gained per lifetap
	local sl = ShadowSeer_GetActivePlayer().spellinfo
	local avg_sp = GetSpellBonusDamage(6)
	if (sl.shadowcount > 0) then
		avg_sp = sl.shadowpower / sl.shadowcount
	end

	local lifetap_mana = (582 + avg_sp * 0.8) * (1 + 0.1 * ShadowSeer_TalentsIn("Improved Life Tap"))
	if (ShadowSeer_TalentsIn("Dark Pact") > 0) then
		lifetap_mana = (582 + avg_sp * 0.8) * 1.2
	end

	-- since extra bolts cost mana, calculate time spent on shadowbolt+life tap to compensate
	local mana_per_bolt = 420 * (1 - 0.01 * ShadowSeer_TalentsIn("Cataclysm"))
	local lifetaps_per_bolt = mana_per_bolt / lifetap_mana
	local lifetap_time_per_bolt = lifetaps_per_bolt * 1.5
	local time_per_bolt = ShadowSeer_GetCastingTime("Shadow Bolt", hr) + lifetap_time_per_bolt
	
	local gained_bolts = gained_time / time_per_bolt
	local gained_damage = floor(gained_bolts * avg_sb_damage + 0.5)

--	ShadowSeer_ShowMessage("totalcasttime=" .. totalcasttime .. " totalnewtime=" .. totalnewtime .. " total gain= " .. gained_time)
--	ShadowSeer_ShowMessage("HASTE: total gained " .. gained_time .. "s, avg_sb_damage = " .. avg_sb_damage .. " sbct = " .. ShadowSeer_GetCastingTime("Shadow Bolt", hr))
--	ShadowSeer_ShowMessage("HASTE: gained_bolts " .. floor(100 * gained_bolts) / 100 .. "s, gained_damage = " .. gained_damage .. " lifetap_time_per_bolt= " .. lifetap_time_per_bolt)
--	ShadowSeer_ShowMessage("HASTE: lifetap_mana= " .. lifetap_mana .. " lifetaps_per_bolt= " .. lifetaps_per_bolt)

	return gained_bolts, gained_damage
end

-- returns amount of talents spent in a talent
-- not very optimal but meh.
function ShadowSeer_TalentsIn(name)
	for i=1,3 do 
		for j=1,GetNumTalents(i) do 
			nameTalent, iconPath, tier, column, currentRank, maxRank, isExceptional, meetsPrereq = GetTalentInfo(i, j)
			if (nameTalent == name) then
				return currentRank
			end
		end
	end
	if (name) then
		ShadowSeer_Log("Error: talent " .. name .. " couldn't be found.")
	end
	return 0
end

function ShadowSeer_GetNumberOfTics(spellname)
-- channeled
	if (spellname == "Drain Life [DOT]") then return 5
	elseif (spellname == "Drain Soul [DOT]") then return 5

-- curses
	elseif (spellname == "Curse of Agony [DOT]") then return 12
	elseif (spellname == "Curse of Doom [DOT]") then return 1

-- dots
	elseif (spellname == "Seed of Corruption [DOT]") then return 6
	elseif (spellname == "Corruption [DOT]") then return 6 -- ignoring bonus
	elseif (spellname == "Unstable Affliction [DOT]") then return 6
	elseif (spellname == "Siphon Life [DOT]") then return 10
	elseif (spellname == "Immolate [DOT]") then return 5

	else return 1
	end
end	

function ShadowSeer_GetSchoolForSpell(spellname)
	spellname = string.gsub(spellname, " %[DOT%]", "")
	
-- channeled
	if (spellname == "Drain Life") then return "Shadow"
	elseif (spellname == "Drain Soul") then return "Shadow"
	elseif (spellname == "Drain Mana") then return "Shadow"

-- curses
	elseif (spellname == "Curse of Agony") then return "Shadow"
	elseif (spellname == "Curse of Shadow") then return "Shadow"
	elseif (spellname == "Curse of the Elements") then return "Shadow"
	elseif (spellname == "Curse of Doom") then return "Shadow"
	elseif (spellname == "Curse of Exhaustion") then return "Shadow"
	elseif (spellname == "Curse of Recklessness") then return "Shadow"
	elseif (spellname == "Curse of Tongues") then return "Shadow"
	elseif (spellname == "Curse of Weakness") then return "Shadow"

-- dots
	elseif (spellname == "Seed of Corruption") then return "Shadow"
	elseif (spellname == "Corruption") then return "Shadow"
	elseif (spellname == "Unstable Affliction") then return "Shadow"
	elseif (spellname == "Siphon Life") then return "Shadow"

-- DD
	elseif (spellname == "Shadowburn") then return "Shadow"
	elseif (spellname == "Shadowfury") then return "Shadow"
	elseif (spellname == "Shadow Bolt") then return "Shadow"
	elseif (spellname == "Death Coil") then return "Shadow"

-- other
	elseif (spellname == "Soulshatter") then return "Shadow"
	elseif (spellname == "Dark Pact") then return "Shadow"
	elseif (spellname == "Life Tap") then return "Shadow"
	
-- fire
	elseif (spellname == "Immolate") then return "Fire"
	elseif (spellname == "Hellfire") then return "Fire"
	elseif (spellname == "Hellfire effect") then return "Fire"
	elseif (spellname == "Incinerate") then return "Fire"
	elseif (spellname == "Rain of Fire") then return "Fire"
	elseif (spellname == "Soul Fire") then return "Fire"
	elseif (spellname == "Searing Pain") then return "Fire"
	elseif (spellname == "Conflagrate") then return "Fire"
	else return "Unknown"
	end
end	

function ShadowSeer_IsDelayedDamageSpell(spellname)
	if (spellname == "Shadow Bolt") then return true
	elseif (spellname == "Death Coil") then return true
	elseif (spellname == "Shadowburn") then return true
	elseif (spellname == "Soul Fire") then return true
	elseif (spellname == "Incinerate") then return true
	else return false
	end
end

function ShadowSeer_SpellCanCrit(spellname)
	if (spellname == "Shadow Bolt") then return true
	elseif (spellname == "Shadowburn") then return true
	elseif (spellname == "Shadowfury") then return true
	elseif (spellname == "Seed of Corruption") then return true
	elseif (spellname == "Immolate") then return true
	elseif (spellname == "Searing Pain") then return true
	elseif (spellname == "Soul Fire") then return true
	elseif (spellname == "Incinerate") then return true
	else return false
	end
end