--
-- REPORTING FUNCTIONS
--

function ShadowSeer_GetPlayerDescription(pl) -- describes a player in text format for reporting
	if (pl.SBcrits == 0) then
		return pl.name
	else
		local dam = floor(pl.ISBcontribution / 6 + 0.5)
		local a = floor(pl.ISBcontribution / 6 / pl.SBcrits + 0.5)
		return pl.name .. ": " .. pl.SBcrits .. " crits, ISB damage contribution: " .. dam .. ". Average per crit " .. a
	end
end

function ShadowSeer_GetSpellDescription(pl, sindex) -- describes a spell list in a text format for reporting
	local line
	local p
	local sl = pl.damagelist[sindex]
	
	if (pl.damagelist[1].totaldamage == 0) then
		return ""
	end
	
	p = floor(100 * sl.totaldamage / pl.damagelist[1].totaldamage + 0.5)
	if (sindex == 1) then
		p = floor(100 * sl.totaldamage / ShadowSeer_GetActiveList()[1].damagelist[1].totaldamage + 0.5)
	end
	
	line = sl.spellname .. " : " .. sl.totaldamage .. " damage. (" .. p .. "%) "

	if (sl.critcount > 0) then
		line = line .. sl.critcount .. "/" .. sl.ticcount .. " " .. "(" .. floor(1000 * sl.critcount / sl.ticcount + 0.5) / 10 .. "% crits) "
	else
		line = line .. sl.ticcount .. " tics "
	end
	
	if (sl.ticcount > 0) then
		line = line .. " Average tic " .. floor(sl.totaldamage / sl.ticcount) .. " "
--		line = line .. " Estimated Average tic " .. floor(ShadowSeer_EstimateAverageTic(sl.spellname)) .. " "
	end

--	if (sl.partialresist > 0) then
--		local r = floor(10000 * sl.partialresist / sl.totaldamage + 0.5) / 100
--		line = line .. sl.partialresist .. " partially resisted (" .. r .. "%) "
--	end
	
	if (sl.ampeddamage > 0) then
		local a = floor(100 * sl.ampeddamage / 6 / (sl.totaldamage / 6 * 5) + 0.5)
		line = line .. a .. "% boosted by ISB"
	end
	return line
end 

--
-- BAR ACTIONS
--
function ShadowSeer_BarOnClick()
	local id = this:GetID()

	-- quick and dirty
	if (arg1 == "LeftButton") then 
		if (ShadowSeer_ViewMode == "PlayerList") then
			ShadowSeer_ReportTo("PARTY", ShadowSeer_Describe_Player())
		elseif (ShadowSeer_ViewMode == "OwnSpells") then
			ShadowSeer_ReportTo("PARTY", ShadowSeer_Describe_Spell())
		elseif (ShadowSeer_ViewMode == "Trinkets") then
			ShadowSeer_ReportTo("PARTY", ShadowSeer_Describe_Trinket())
		elseif (ShadowSeer_ViewMode == "OwnResists") then
			-- NYI
		end
	elseif (arg1 == "RightButton") then
		if (ShadowSeer_ViewMode == "PlayerList") then
			ShadowSeer_Selected_Index = id
		else
			ShadowSeer_Selected_Index = -1
		end
		ToggleDropDownMenu(1, nil, ShadowSeerFrameDropDown, this:GetName(), 55, 10)
	end
end

function ShadowSeer_BarOnLeave()
	ShadowSeer_Tooltip:Hide()
	ShadowSeer_Tooltip_Visible = false
end

-- TOOLTIP --
function ShadowSeer_BarOnEnter()
	if (ShadowSeer_ViewMode == "PlayerList") then
		ShadowSeer_SetTooltipText(ShadowSeer_Describe_Player())
	elseif (ShadowSeer_ViewMode == "OwnSpells") then
		ShadowSeer_SetTooltipText(ShadowSeer_Describe_Spell())
	elseif (ShadowSeer_ViewMode == "Trinkets") then
		ShadowSeer_SetTooltipText(ShadowSeer_Describe_Trinket())
	elseif (ShadowSeer_ViewMode == "OwnResists") then
		ShadowSeer_Tooltip_OwnResist()
	else
		if (ShadowSeer_ViewMode) then
			ShadowSeer_Log("Error: ViewMode has incorrect value: " .. ShadowSeer_ViewMode)
		else
			ShadowSeer_Log("Error: ViewMode is nil")
		end
	end
end

function ShadowSeer_Describe_Trinket()
	local i = this:GetID()
	local al = ShadowSeer_GetActiveList()
	local pl = ShadowSeer_GetPlayer(al, UnitName("player"))
	local bi = pl.proctable[i]
	local lp = floor(10*(GetTime() - bi.lastprocced))/10
	local ad = floor(10* bi.spellpower * bi.spellcount / ShadowSeer_GetActivePlayer().spellinfo.spellcount) / 10

	local r,g,b = ShadowSeer_GetColorForName(bi.name)
	local trinkettip = {}
	ShadowSeer_AddLineTo(trinkettip, bi.name, r, g, b)
	
	ShadowSeer_AddLineTo(trinkettip, " ")

	local sinfo = ShadowSeer_GetActivePlayer().spellinfo
	local avg_firepower = floor(sinfo.firepower / sinfo.firecount + 0.5)
	local avg_shadowpower = floor(sinfo.shadowpower / sinfo.shadowcount + 0.5)
	local avg_spellpower = floor(sinfo.spellpower / sinfo.spellcount + 0.5)

	local totalcasts = sinfo.spellcount
	if (bi.school == "All") then
		ShadowSeer_AddLineTo(trinkettip, "Grants " .. bi.spellpower .. " spellpower for " .. floor(bi.duration + 0.5) .. " seconds.", 1, 1, 1)
	elseif (bi.school == "Fire") then
		ShadowSeer_AddLineTo(trinkettip, "Grants " .. bi.spellpower .. " spellpower to Fire spells for " .. floor(bi.duration + 0.5) .. " seconds.", 1, 0.3, 0.3)
		totalcasts = sinfo.firecount
	elseif (bi.school == "Shadow") then 
		ShadowSeer_AddLineTo(trinkettip, "Grants " .. bi.spellpower .. " spellpower to Shadow spells for " .. floor(bi.duration + 0.5) .. " seconds.", 0.3, 0.3, 1)
		totalcasts = sinfo.shadowcount
	end

	local spells_affected_ratio = bi.spellcount / max(totalcasts, 1)
	local stack_ratio = bi.stackcount / max(totalcasts, 1)
	local formatted_ratio = floor(spells_affected_ratio * 10000) / 100
	local average_spellpower_bonus = floor(bi.spellpower * stack_ratio * 10) / 10
		
	ShadowSeer_AddLineTo(trinkettip, " ")
	ShadowSeer_AddLineTo(trinkettip, "Number of procs: " .. bi.proccount .. ".", 1, 1, 1)
	ShadowSeer_AddLineTo(trinkettip, "It affected " .. bi.spellcount .. " / " .. totalcasts .. " offensive spells. (" .. formatted_ratio .. "%)", 1, 1, 1)
	ShadowSeer_AddLineTo(trinkettip, "Average contribution: " ..average_spellpower_bonus .. " spell power.", 1, 1, 1)
	ShadowSeer_AddLineTo(trinkettip, "Only spellpower based spells are counted towards these totals", 0.75, 0.75, 0.75)

	if (bi.stackcount ~= bi.spellcount) then
		ShadowSeer_AddLineTo(trinkettip, "Average amount of stacks active : " .. floor(100 * bi.stackcount / bi.spellcount + 0.5) / 100, 1, 1, 1)
	end
	
	if (bi.cooldown ~= -1) then
		ShadowSeer_AddLineTo(trinkettip, "Minimum time between procs was " .. floor(bi.cooldown + 0.5) .. "s.", 1, 1, 1)
	end

	ShadowSeer_AddLineTo(trinkettip, " ")

	if (sinfo.spellcount > 0) then
		local avg_spellpower = floor(sinfo.spellpower / max(1, sinfo.spellcount) + 0.5)
		
		ShadowSeer_AddLineTo(trinkettip, "Average Spell power was " .. avg_spellpower .. ". (" .. sinfo.spellcount .. " spells).", 1, 1, 1)
		if (sinfo.shadowcount > 0) then
			local current_shadowpower = GetSpellBonusDamage(6)
			local avg_shadowpower = floor(sinfo.shadowpower / sinfo.shadowcount + 0.5)
			local diff_shadowpower = avg_shadowpower - current_shadowpower
			local shadowratio = floor(10000 * sinfo.shadowcount / sinfo.spellcount + 0.5) / 100
			ShadowSeer_AddLineTo(trinkettip, " ")
			ShadowSeer_AddLineTo(trinkettip, "Average Shadow spellpower was " .. avg_shadowpower .. " on " .. sinfo.shadowcount .. " Shadow spells (" .. shadowratio .. "%)", 0.25, 0.25, 1)
			ShadowSeer_AddLineTo(trinkettip, "Currently you have " .. current_shadowpower .. ". (" .. diff_shadowpower .. " difference)", 0.25, 0.25, 1)
		end
		
		if (sinfo.firecount > 0) then
			local current_firepower = GetSpellBonusDamage(3)
			local avg_firepower = floor(sinfo.firepower / sinfo.firecount + 0.5)
			local diff_firepower = avg_firepower - current_firepower
			local fireratio = floor(10000 * sinfo.firecount / sinfo.spellcount + 0.5) / 100
			ShadowSeer_AddLineTo(trinkettip, " ")
			ShadowSeer_AddLineTo(trinkettip, "Average Fire spellpower was " .. avg_firepower .. " on " .. sinfo.firecount .. " Fire spells (" .. fireratio .. "%)", 1, 0.2, 0.2)
			ShadowSeer_AddLineTo(trinkettip, "Currently you have " .. current_firepower .. ". (" .. diff_firepower .. " difference)", 1, 0.2, 0.2)
		end
	end
	return trinkettip
end

function ShadowSeer_Describe_Player()
	local i = this:GetID()
	local pl = ShadowSeer_GetActiveList()[i]

	local playertip = {}
	if (i == 1) then
		ShadowSeer_AddLineTo(playertip, ShadowSeer_GetPlayerDescription(pl), 0.8, 0.8, 0.8)
	else
		local color = RAID_CLASS_COLORS[strupper(pl.class)]
		ShadowSeer_AddLineTo(playertip, ShadowSeer_GetPlayerDescription(pl), color.r, color.g, color.b)
	end
	
	ShadowSeer_AddLineTo(playertip, ShadowSeer_GetSpellDescription(pl, 1), 1, 1, 1)
	ShadowSeer_AddLineTo(playertip, "")
	for j = 2, table.getn(pl.damagelist) do
		if (ShadowSeer_GetSchoolForSpell(pl.damagelist[j].spellname) == "Shadow") then
			ShadowSeer_AddLineTo(playertip, ShadowSeer_GetSpellDescription(pl, j), 1, 1, 0.8)
		end
	end

	return playertip
end

function ShadowSeer_Describe_Spell()
	local i = this:GetID()
	local pl = ShadowSeer_GetActivePlayer()
	local dl = pl.damagelist
	local rt = pl.resulttable

	local spelltip = {}
	
	local r,g,b = ShadowSeer_GetColorForSpell(dl[i].spellname)
	if (i == 1) then
		ShadowSeer_AddLineTo(spelltip, dl[i].spellname, r, g, b)
	else
		local a = floor(100 * dl[i].totaldamage / dl[1].totaldamage + 0.5)
		ShadowSeer_AddLineTo(spelltip, dl[i].spellname .. " (" .. a .. "%)", r, g, b)
	end

	ShadowSeer_AddLineTo(spelltip, " ", 1, 1, 1)

	if (dl[i].ticcount > 0) then
		-- Total Damage: x (y% boosted by ISB) on z hits
		local line = "Total Damage: " .. dl[i].totaldamage
		
		if (dl[i].ampeddamage > 0) then
			local a = floor(100 * dl[i].ampeddamage / 6 / (dl[i].totaldamage / 6 * 5) + 0.5)
			line = line .. " (" .. a .. "% boosted by ISB)"
		end
		local avg = floor(dl[i].totaldamage / max(1, dl[i].ticcount) + 0.5)
		line = line .. " on " .. dl[i].ticcount .. " hits. (" .. avg .. " average)"
		
		if (dl[i].spellname ~= "Hellfire Effect" and dl[i].spellname ~= "Rain of Fire") then
			ShadowSeer_AddLineTo(spelltip, line, 1, 1, 1)
			ShadowSeer_AddLineTo(spelltip, " ", 1, 1, 1)
		end

		local hitcount = 0
		local castcount = 0
		local sname = string.gsub(dl[i].spellname, " %[DOT%]", "")
		for j = 1,table.getn(rt) do 
			if (rt[j].spellname == sname) then
				for k = 1,table.getn(rt[j].resultlist) do 
					hitcount = hitcount + rt[j].resultlist[k].castcount - rt[j].resultlist[k].resistcount
					castcount = castcount + rt[j].resultlist[k].castcount
				end
			end
		end

		-- highest hit: x
		ShadowSeer_AddLineTo(spelltip, "Highest hit: " .. dl[i].maxdamage, 1, 1, 1)
		ShadowSeer_AddLineTo(spelltip, " ")
		
	-- give spell result details
		if (i == 1) then -- first lists all
			ShadowSeer_AddLineTo(spelltip, "Breakdown: ")
			for j = 1, table.getn(pl.resulttable) do
				ShadowSeer_AddLineTo(spelltip, pl.resulttable[j].spellname, 1, 0.75, 0.75)
				ShadowSeer_DescribeResult(spelltip, pl.resulttable[j])
			end
		elseif (dl[i].spellname ~= "Hellfire Effect") then
			-- resist rate: x %
			local rrate = floor(0.5 + (1 - hitcount / castcount) * 10000) / 100
			ShadowSeer_AddLineTo(spelltip, "Resist ratio: " .. (castcount - hitcount) .. " / " .. castcount .. " (" .. rrate .. "%)", 1, 1, 1)
			ShadowSeer_AddLineTo(spelltip, " ")
			local sname = string.gsub(dl[i].spellname, " %[DOT%]", "")
			
			for j = 1, table.getn(pl.resulttable) do
				if (pl.resulttable[j].spellname == sname) then
					ShadowSeer_DescribeResult(spelltip, pl.resulttable[j])
				end
			end
		end
		ShadowSeer_AddLineTo(spelltip, " ", 1, 1, 1)
		ShadowSeer_AddLineTo(spelltip, " ", 1, 1, 1)
		
		local average_damage_per_cast = 0
		local castcount = 0

		if (string.find(dl[i].spellname, "%[DOT%]")) then
			local average_tics_per_cast = floor(100 * dl[i].ticcount / max(hitcount, 1)) / 100
			ShadowSeer_AddLineTo(spelltip, "Total tics: " .. dl[i].ticcount .. " average tics per cast: " .. average_tics_per_cast, 1, 1, 1)
			average_damage_per_cast = floor(dl[i].totaldamage / hitcount + 0.5)
			ShadowSeer_AddLineTo(spelltip, "Average damage per unresisted cast: " .. average_damage_per_cast, 1, 1, 1)
			ShadowSeer_AddLineTo(spelltip, " ", 1, 1, 1)
			local dotname = string.gsub(dl[i].spellname, " %[DOT%]", "")
			for j = 1,table.getn(rt) do 
				if (rt[j].spellname == dotname) then
					for k = 1,table.getn(rt[j].resultlist) do 
						castcount = castcount + rt[j].resultlist[k].castcount - rt[j].resultlist[k].resistcount
					end
				end
			end
		elseif (dl[i].spellname == "Seed [DETONATION]") then
			for j = 1,table.getn(rt) do 
				if (rt[j].spellname == "Seed of Corruption") then
					for k = 1,table.getn(rt[j].resultlist) do 
						castcount = castcount + rt[j].resultlist[k].castcount - rt[j].resultlist[k].resistcount
					end
				end
			end
			local average_tics_per_cast = floor(100 * dl[i].ticcount / max(castcount, 1)) / 100
			ShadowSeer_AddLineTo(spelltip, "Total hits: " .. dl[i].ticcount .. " Unresisted Cast count: " .. castcount .. " average hits per Seed cast: " .. average_tics_per_cast, 1, 1, 1)
			ShadowSeer_AddLineTo(spelltip, " ", 1, 1, 1)
		elseif (dl[i].spellname == "Hellfire Effect") then
			local selftics = 0
			for j = 1,table.getn(rt) do 
				if (rt[j].spellname == "Hellfire") then
					for k = 1,table.getn(rt[j].resultlist) do 
						castcount = rt[j].resultlist[k].castcount
						selftics = rt[j].resultlist[k].selfticcount
					end
				end
			end
			local ticcount = 0
			for j = 1,table.getn(dl) do 
				if (dl[j].spellname == "Hellfire Effect") then
					ticcount = dl[j].ticcount
				end
			end
			local average_tics_per_cast = floor(100 * ticcount / max(castcount, 1)) / 100
			ShadowSeer_AddLineTo(spelltip, "Total hits: " .. ticcount .. "  Cast count: " .. castcount .. " average hits per Hellfire cast: " .. average_tics_per_cast, 1, 1, 1)
			ShadowSeer_AddLineTo(spelltip, " ", 1, 1, 1)
			
			local casttime = selftics / (1 + GetCombatRating(CR_HASTE_SPELL) / 15.77 / 100)
			local edps = floor(dl[i].totaldamage / casttime + 0.5)
			ShadowSeer_AddLineTo(spelltip, "Estimated time spent channeling: " .. floor(100 * casttime + 0.5) / 100 .. "s for " .. edps .. " dps" , 1, 1, 1)
			
		elseif (dl[i].spellname == "Rain of Fire") then
			for j = 1,table.getn(rt) do 
				if (rt[j].spellname == "Rain of Fire") then
					castcount = rt[j].resultlist[1].castcount
				end
			end
			ShadowSeer_AddLineTo(spelltip, "Average Total Damage Per Rain cast: " .. floor(dl[i].totaldamage / castcount + 0.5))
			ShadowSeer_AddLineTo(spelltip, "Average Tics per cast: " .. floor(100*dl[i].ticcount / castcount + 0.5) / 100)
			local ctime = 8 / (1 + GetCombatRating(CR_HASTE_SPELL) / 15.77 / 100)
			ShadowSeer_AddLineTo(spelltip, "Average Damage Per Casting Time: " .. floor(dl[i].totaldamage / castcount / ctime + 0.5) .. " dpcs, assuming fully channeled Rain of Fire.")
			ShadowSeer_AddLineTo(spelltip, " ", 1, 1, 1)
		end
	
		-- Average hit: x 
		-- Average crit: z (z% crit ratio, total z1% damage)
		local avg_hit_with_isb = avg

		if (dl[i].critcount > 0) then
			local cchance = dl[i].critcount / dl[i].ticcount
			local crate = floor(10000 * cchance + 0.5) / 100
			ShadowSeer_AddLineTo(spelltip, "Average damage : " .. avg .. " with " .. dl[i].critcount .. "/" .. dl[i].ticcount .. " crits/hits (" .. crate .. "% crit chance)", 1, 1, 1)
			-- Average noncrit: y (z% noncrit ratio, total z1% damage)
			local avgnc = floor((dl[i].totaldamage - dl[i].totalcritdamage) / max(1, dl[i].ticcount - dl[i].critcount) + 0.5)
			local dratio = floor(10000 * dl[i].totalcritdamage / dl[i].totaldamage + 0.5) / 100
--			ShadowSeer_AddLineTo(spelltip, "Average NonCrit: " .. avgnc .. " (" .. (100-dratio) .. "% of the damage)", 0.5, 0.5, 0.5)
			-- Average crit: z (z% noncrit ratio, total z1% damage)
			local avgc = floor(dl[i].totalcritdamage / dl[i].critcount + 0.5)
--			ShadowSeer_AddLineTo(spelltip, "Average Crit: " .. avgc .. " (" .. dratio .. "% of the damage)", 0.5, 0.5, 0.5)

			-- ISB
			if (dl[i].spellname == "Shadow Bolt" and ShadowSeer_TalentsIn("Improved Shadow Bolt") == 5) then
				local a = floor(pl.ISBcontribution / 6 / pl.SBcrits + 0.5)
				ShadowSeer_AddLineTo(spelltip, "Average Crit including ISB: " .. floor(avgc + a + 0.5), 1, 1, 1)
				
				avg_hit_with_isb = floor((1-cchance)*avgnc + cchance * (avgc + a) + 0.5)
				ShadowSeer_AddLineTo(spelltip, "Average Bolt damage including ISB: " .. avg_hit_with_isb, 1, 1, 1)
			end
		elseif (i > 1) then
			ShadowSeer_AddLineTo(spelltip, "Average damage : " .. avg .. " (" .. dl[i].ticcount .. " hits)", 1, 1, 1)
			ShadowSeer_AddLineTo(spelltip, " ", 1, 1, 1)
		end

		-- estimate damage-per-casting-time here
		local average_damage_per_cast = dl[i].totaldamage / dl[i].ticcount
		if (dl[i].spellname == "Shadow Bolt") then
			average_damage_per_cast = avg_hit_with_isb
		elseif (dl[i].spellname == "Immolate" or dl[i].spellname == "Immolate [DOT]") then
			local damage_direct = 0
			local damage_dot = 0
			local hitcount = 0
			for z = 1,table.getn(dl) do
				if (dl[z].spellname == "Immolate") then 
					damage_direct = dl[z].totaldamage
					hitcount = dl[z].ticcount
				elseif (dl[z].spellname == "Immolate [DOT]") then 
					damage_dot = dl[z].totaldamage
				end
			end
--			ShadowSeer_AddLineTo(spelltip, "Immolation Tic Total: " .. damage_dot)
--			ShadowSeer_AddLineTo(spelltip, "Immolation Impact Total: " .. damage_direct)
--			ShadowSeer_AddLineTo(spelltip, "Immolation Total: " .. damage_direct + damage_dot)
--			ShadowSeer_AddLineTo(spelltip, "Immolation count: " .. hitcount)
			average_damage_per_cast = (damage_direct + damage_dot) / hitcount
		elseif (string.find(dl[i].spellname, "%[DOT%]") or dl[i].spellname == "Seed [DETONATION]" or dl[i].spellname == "Rain of Fire") then
			average_damage_per_cast = dl[i].totaldamage / castcount
		end
		
		if (i > 1 and dl[i].spellname ~= "Hellfire Effect" and dl[i].spellname ~= "Rain of Fire") then
			ShadowSeer_AddLineTo(spelltip, "Average Total Damage Per Succesful cast: " .. floor(average_damage_per_cast + 0.5))
			ShadowSeer_AddLineTo(spelltip, "Average Total Damage Per Casting Time: " .. floor(average_damage_per_cast / ShadowSeer_GetCastingTime(dl[i].spellname) + 0.5) .. "dpcs (assuming no resists)")
			ShadowSeer_AddLineTo(spelltip, " ", 1, 1, 1)
		end
		
		-- partial resisted damage: x (y% of total)
		if (dl[i].partialresist > 0) then
			avg = floor(10000 * dl[i].partialresist / (dl[i].totaldamage + dl[i].partialresist) + 0.5) / 100
			ShadowSeer_AddLineTo(spelltip, "Partially resisted damage:" .. dl[i].partialresist .. " (" .. avg .. "%)", 1, 1, 1)
			ShadowSeer_AddLineTo(spelltip, " ", 1, 1, 1)
		end
	end

	local spower = GetSpellBonusDamage(6)
	if (ShadowSeer_GetSchoolForSpell(dl[i].spellname) == "Fire") then
		spower = GetSpellBonusDamage(3)
	end

	if (i > 1 and ShadowSeer_EstimateAverageTicWithSpellpower(dl[i].spellname, spower) > 0) then
		local tics = ShadowSeer_GetNumberOfTics(dl[i].spellname)
		
		local old_estimated_tic = ShadowSeer_EstimateAverageTicWithSpellpower(dl[i].spellname, spower)
		local new_estimated_tic = floor(ShadowSeer_EstimateAverageTicWithSpellpower(dl[i].spellname, spower + 100)+0.5)
		local gratio = floor((new_estimated_tic/max(1, old_estimated_tic) - 1) * 10000) / 100
		if (tics > 1) then
			ShadowSeer_AddLineTo(spelltip, "Theoretical average tic with no modifiers: " .. floor(old_estimated_tic + 0.5))
			ShadowSeer_AddLineTo(spelltip, "Theoretical average tic with 100 extra spelldamage: " .. floor(new_estimated_tic + 0.5))
			ShadowSeer_AddLineTo(spelltip, "Theoretical average full damage: " .. floor(old_estimated_tic * tics + 0.5))
			ShadowSeer_AddLineTo(spelltip, "Theoretical average full damage with 100 extra spelldamage: " .. floor(new_estimated_tic * tics + 0.5) .. " (" .. gratio .. "% more)")
			ShadowSeer_AddLineTo(spelltip, " ")
		else
			ShadowSeer_AddLineTo(spelltip, "Theoretical average hit with no modifiers: " .. floor(old_estimated_tic + 0.5))
			ShadowSeer_AddLineTo(spelltip, "Theoretical average hit with 100 extra spelldamage: " .. floor(new_estimated_tic + 0.5) .. " (" .. gratio .. "% more)")
			ShadowSeer_AddLineTo(spelltip, " ")
		end
	elseif (i == 1 and UnitClass("player") == "Warlock") then
		-- count bonus damage
		local bonusd = ShadowSeer_EstimateAddedDamageWithSpellPower(pl.damagelist, 100)
		local gain = floor(10000 * bonusd / max(1, pl.damagelist[1].totaldamage) + 0.5) / 100
		
		ShadowSeer_AddLineTo(spelltip, "100 extra spelldamage would have gotten you " .. bonusd .. " more damage (" .. gain .. "%)")
		
		-- count crit damage
		bonusd = ShadowSeer_EstimateAddedDamageWithCritRating(pl.damagelist, 100)
		gain = floor(10000 * bonusd / max(1, pl.damagelist[1].totaldamage) + 0.5) / 100
		
		ShadowSeer_AddLineTo(spelltip, "100 extra crit rating would have gotten you " .. bonusd .. " more damage (" .. gain .. "%)")
		
		-- count +hit damage
		if (ShadowSeer_HitRatingNeededTillCap() >= 100) then
			bonusbolts, bonusd = ShadowSeer_EstimateTimeAndDamageGainedWithHitRating(100)
			gain = floor(10000 * bonusd / max(1, pl.damagelist[1].totaldamage) + 0.5) / 100
			ShadowSeer_AddLineTo(spelltip, "100 extra hit rating would have gotten you " .. bonusd .. " more damage (" .. gain .. "%) due to gaining " .. bonusbolts .. " Shadow Bolts")
		else
			local hrn = floor(ShadowSeer_HitRatingNeededTillCap()+0.5)
			bonusbolts, bonusd = ShadowSeer_EstimateTimeAndDamageGainedWithHitRating(hrn)
			gain = floor(10000 * bonusd / max(1, pl.damagelist[1].totaldamage) + 0.5) / 100
			ShadowSeer_AddLineTo(spelltip, hrn .. " extra hit rating makes you hit the resist cap, getting you " .. bonusd .. " more damage (" .. gain .. "%) due to gaining " .. bonusbolts .. " Shadow Bolts")
		end

		-- count +haste damage
		bonusbolts, bonusd = ShadowSeer_EstimateTimeAndDamageGainedWithHasteRating(100)
		gain = floor(10000 * bonusd / max(1, pl.damagelist[1].totaldamage) + 0.5) / 100
		bonusbolts = floor(bonusbolts * 10 + 0.5)/10
		ShadowSeer_AddLineTo(spelltip, "100 extra haste rating would have gotten you " .. bonusd .. " more damage (" .. gain .. "%) due to gaining " .. bonusbolts .. " Shadow Bolts.")
	end

	return spelltip
end

function ShadowSeer_DescribeResult(tip, rt)
	for k = 1, table.getn(rt.resultlist) do
		rl = rt.resultlist[k]
		if (rl.castcount > 0) then -- no need to monitor life tap / dark pact for resists
			local line = "Nontargeted : " .. rl.castcount .. " casts."
			if (rl.level) then
				local failrate = floor(10000 * rl.resistcount / rl.castcount + 0.5) / 100
				line = "vs. lvl " .. rl.level .. " : " .. rl.resistcount .. "/" .. rl.castcount .. " resists (" .. failrate .. "%)"
			end
				
			if (rl.critcount > 0) then
				local landcount = rl.castcount - rl.resistcount
				local critrate = floor(10000 * rl.critcount / landcount + 0.5) / 100
				line = line .. "  " .. rl.critcount .. "/" .. landcount .. " crits (" .. critrate .. "%)"
			end
					
			if (rl.totaldamage > 0) then
				local avg = floor(rl.totaldamage / rl.castcount + 0.5) 
				line = line .. " total " .. rl.totaldamage .. " damage, average " .. avg .. " per cast."
			end
			ShadowSeer_AddLineTo(tip, line, 0.8, 0.8, 0.8)
		end
	end
end

--
-- REDO BARS
--
-- refreshes bars after an update.
function ShadowSeer_RedoBars()
	if (ShadowSeer_ViewMode == "PlayerList") then
		ShadowSeer_RedoBars_PlayerList(ShadowSeer_GetActiveList())
	elseif (ShadowSeer_ViewMode == "OwnSpells") then
		ShadowSeer_RedoBars_OwnSpells(ShadowSeer_GetActiveList())
	elseif (ShadowSeer_ViewMode == "Trinkets") then
		ShadowSeer_RedoBars_Trinkets(ShadowSeer_GetActiveList())
	elseif (ShadowSeer_ViewMode == "OwnResists") then
		ShadowSeer_RedoBars_OwnResists(ShadowSeer_GetActiveList())
	else
		if (ShadowSeer_ViewMode) then
			ShadowSeer_Log("Error: ViewMode has incorrect value: " .. ShadowSeer_ViewMode)
		else
			ShadowSeer_Log("Error: ViewMode is nil")
		end
	end
end
	
function ShadowSeer_RedoBars_PlayerList(pl)
	ShadowSeerFrame:SetHeight(13 + max(5, table.getn(pl)+1) * ShadowSeer_BARHEIGHT)
	
	local maxval = max(ShadowSeer_PlayerList_AllCombats[1].damagelist[1].totaldamage, 1)
	for i=1,min(ShadowSeer_MAX_BARS, table.getn(pl)) do
	-- calculate max value quick
		ShadowSeer_bars[i]:SetMinMaxValues(0, maxval)
		ShadowSeer_bars[i]:SetValue(pl[i].damagelist[1].totaldamage)
	
		if (i == 1) then 
			ShadowSeer_bars[i]:SetStatusBarColor(0.8, 0.8, 0.8)
		else
			local color = RAID_CLASS_COLORS[strupper(pl[i].class)]
			ShadowSeer_bars[i]:SetStatusBarColor(color.r, color.g, color.b)
		end
	
		ShadowSeer_text[i]:SetText(pl[i].name)
		ShadowSeer_text[i]:Show()
		ShadowSeer_bars[i]:Show()
	end
	for i=table.getn(pl)+1, ShadowSeer_MAX_BARS do
		ShadowSeer_bars[i]:Hide()
		ShadowSeer_text[i]:Hide()
	end
end
	
function ShadowSeer_RedoBars_OwnSpells(pl)
	local dl = ShadowSeer_GetPlayer(pl, UnitName("player")).damagelist
	
	ShadowSeerFrame:SetHeight(13 + max(5, table.getn(dl)+1) * ShadowSeer_BARHEIGHT)
	
	local maxval = max(dl[1].totaldamage, 1)
	for i=1,min(ShadowSeer_MAX_BARS, table.getn(dl)) do
	-- calculate max value quick
		ShadowSeer_bars[i]:SetMinMaxValues(0, maxval)
		ShadowSeer_bars[i]:SetValue(dl[i].totaldamage)
	
		if (i == 1) then 
			ShadowSeer_bars[i]:SetStatusBarColor(0.8, 0.8, 0.8)
		else
			local r,g,b = ShadowSeer_GetColorForSpell(dl[i].spellname)
			ShadowSeer_bars[i]:SetStatusBarColor(r, g, b)
		end
	
		ShadowSeer_text[i]:SetText(dl[i].spellname)
		ShadowSeer_text[i]:Show()
		ShadowSeer_bars[i]:Show()
	end
	for i=table.getn(dl)+1, ShadowSeer_MAX_BARS do
		ShadowSeer_bars[i]:Hide()
		ShadowSeer_text[i]:Hide()
	end
end

function ShadowSeer_RedoBars_Trinkets(pl)
	local bl = ShadowSeer_GetPlayer(pl, UnitName("player")).proctable
	
	ShadowSeerFrame:SetHeight(13 + max(5, table.getn(bl)+2) * ShadowSeer_BARHEIGHT)

	local totalspells = max(1, ShadowSeer_GetActivePlayer().spellinfo.spellcount)

	-- determine total value as maxval
	local maxval = 0
	for i=1,table.getn(bl) do
		maxval = maxval + bl[i].spellpower * bl[i].spellcount / totalspells
	end
	maxval = max(maxval, 1)
	
	for i=1,min(ShadowSeer_MAX_BARS, table.getn(bl)) do
		ShadowSeer_bars[i]:SetMinMaxValues(0, maxval)
		ShadowSeer_bars[i]:SetValue(bl[i].spellpower * bl[i].spellcount / totalspells)

		local r,g,b = ShadowSeer_GetColorForName(bl[i].name)
		ShadowSeer_bars[i]:SetStatusBarColor(r, g, b)
	
		ShadowSeer_text[i]:SetText(bl[i].name)
		ShadowSeer_text[i]:Show()
		ShadowSeer_bars[i]:Show()
	end
	for i=table.getn(bl)+1, ShadowSeer_MAX_BARS do
		ShadowSeer_bars[i]:Hide()
		ShadowSeer_text[i]:Hide()
	end
end

-- field per spell, which combines totals
function ShadowSeer_RedoBars_OwnResists(pl)
	local rl = ShadowSeer_GetPlayer(pl, UnitName("player")).resulttable
	
	ShadowSeerFrame:SetHeight(13 + max(5, table.getn(rl)+1) * ShadowSeer_BARHEIGHT)
	
	local maxval = 0
	local casttotals = {}
	
	for i=1,table.getn(rl) do
		table.insert(casttotals, 0)
		for j=1,table.getn(rl[i].resultlist) do 
			maxval = maxval + rl[i].resultlist[j].castcount
			casttotals[i] = casttotals[i] + rl[i].resultlist[j].castcount
		end
	end
	maxval = max(maxval, 1)

	for i=1,min(ShadowSeer_MAX_BARS, table.getn(rl)) do
	-- calculate max value quick
		ShadowSeer_bars[i]:SetMinMaxValues(0, maxval)
		ShadowSeer_bars[i]:SetValue(casttotals[i])
	
		local r,g,b = ShadowSeer_GetColorForSpell(rl[i].spellname)
		ShadowSeer_bars[i]:SetStatusBarColor(r, g, b)
	
		ShadowSeer_text[i]:SetText(rl[i].spellname)
		ShadowSeer_text[i]:Show()
		ShadowSeer_bars[i]:Show()
	end
	for i=table.getn(rl)+1, ShadowSeer_MAX_BARS do
		ShadowSeer_bars[i]:Hide()
		ShadowSeer_text[i]:Hide()
	end
end

--
-- UI functions
--

--
-- Title button
--
function ShadowSeer_TitleButton_OnClick()
	ShadowSeer_Selected_Index = -1
	ToggleDropDownMenu(1, nil, ShadowSeerFrameDropDown, "ShadowSeerFrame_TitleButton", 55, 10)
end

function ShadowSeer_TitleButton_OnLoad()
	this:SetBackdropColor(SHADOWSEER_BGCOLOR_RED, SHADOWSEER_BGCOLOR_GREEN, SHADOWSEER_BGCOLOR_BLUE, SHADOWSEER_BGCOLOR_ALPHA)
	ShadowSeer_TitleButtonText:SetText("ShadowSeer " .. SHADOWSEER_VERSION)
	this:RegisterForClicks("LeftButtonDown", "RightButtonDown")
end

function ShadowSeer_Initialise_Frame()
	this:SetBackdropColor(SHADOWSEER_BGCOLOR_RED, SHADOWSEER_BGCOLOR_GREEN, SHADOWSEER_BGCOLOR_BLUE, SHADOWSEER_BGCOLOR_ALPHA)

	local name = this:GetName()
	ShadowSeer_Main_Frame = this
	for i = 1,ShadowSeer_MAX_BARS do
		ShadowSeer_bars[i] = getglobal(name.."Bar"..i)
		ShadowSeer_bars[i]:SetID(i)
		ShadowSeer_text[i] = getglobal(name.."Text"..i)
		ShadowSeer_text[i]:SetText("player"..i)

		local itemButton = ShadowSeer_bars[i]
		local itemText = ShadowSeer_text[i]

		itemButton:Hide()

		itemButton:SetWidth(ShadowSeer_BARWIDTH)
		itemButton:SetHeight(ShadowSeer_BARHEIGHT)

		local x = 5
		local y = 9 - i * ShadowSeer_BARHEIGHT
		itemButton:SetPoint("TOPLEFT", name, "TOPLEFT", x, y)
		itemButton:SetStatusBarColor(0, 1.0, 1.00)
		itemButton:SetMinMaxValues(0, 1)
		itemButton:SetValue(1)
		SetTextStatusBarText(itemButton, itemText)
		HideTextStatusBarText(itemButton)

		itemText:SetPoint("CENTER", itemButton:GetName(), "CENTER", 0, 0)
		itemText:SetPoint("LEFT", itemButton:GetName(), "LEFT", 0, 0)
		itemText:SetPoint("TOP", itemButton:GetName(), "TOP", 0, 0)
		itemText:SetPoint("RIGHT", itemButton:GetName(), "RIGHT", 0, 0)
		itemText:SetPoint("BOTTOM", itemButton:GetName(), "BOTTOM", 0, 0)
		itemText:SetJustifyH("CENTER")
	end
	if (ShadowSeer_Visible) then
		this:Show()
	else
		this:Hide()
	end
end

--
-- Dropdown Menu
--
function ShadowSeer_FrameDropDown_OnLoad()
	UIDropDownMenu_Initialize(this, ShadowSeer_FrameDropDown_Initialise, "MENU")
end

function ShadowSeer_FrameDropDown_Initialise(level)
	if (level == 1) then
		ShadowSeer_FrameDropDown_InitMainMenu()
	elseif (level == 2) then
		ShadowSeer_FrameDropDown_InitSubMenu()
	end
end

function ShadowSeer_FrameDropDown_InitMainMenu()
	local info = {}
	info.text = "ShadowSeer Options"
	info.notClickable = 1
	info.isTitle = 1
	UIDropDownMenu_AddButton(info, 1)

	info = {}
	info.text = "Hide ShadowSeer"
	info.value = 1
	info.func = ShadowSeer_FrameDropDownMenu_Hide
	UIDropDownMenu_AddButton(info, 1)
	
	info = {}
	info.text = "Disable ShadowSeer"
	info.value = 2
	info.func = ShadowSeer_FrameDropDownMenu_Disable
	
	if (not ShadowSeer_Enabled) then
		info.text = "Enable ShadowSeer"
		info.func = ShadowSeer_FrameDropDownMenu_Enable
	end
	UIDropDownMenu_AddButton(info, 1)
	
	info = {}
	info.text = "Clear List"
	info.value = 3
	info.func = ShadowSeer_FrameDropDownMenu_Clear
	UIDropDownMenu_AddButton(info, 1)	

	info = {}
	if (ShadowSeer_ViewingAllCombats) then
		info.text = "View Data for last combat only"
	else
		info.text = "View Data for all combats"
	end
	info.value = 4
	info.func = ShadowSeer_FrameDropDownMenu_Toggle
	UIDropDownMenu_AddButton(info, 1)	

--[[
	if (ShadowSeer_Selected_Index > -1) then
		local name = ShadowSeer_GetActiveList()[ShadowSeer_Selected_Index].name
		local class = ShadowSeer_GetActiveList()[ShadowSeer_Selected_Index].class

		if ((class == "Warlock") or not (name == UnitName("player"))) then
			info = {}
			info.text = "Mark " .. name .. " as having ISB"
			if (ShadowSeer_WarlockHasISB(name)) then
				info.text = "Mark " .. name .. " as not having ISB"
			end
			info.value = 5
			info.func = ShadowSeer_FrameDropDownMenu_MarkISB

			UIDropDownMenu_AddButton(info, 1)	
		end
	end
]]--	
	info = {}
	info.text = "View raid Shadow damage"
	info.value = 6
	info.func = ShadowSeer_FrameDropDownMenu_ViewRaid
	UIDropDownMenu_AddButton(info, 1)	
	
	info = {}
	info.text = "View your spell damage"
	info.value = 7
	info.func = ShadowSeer_FrameDropDownMenu_ViewOwnDamage
	UIDropDownMenu_AddButton(info, 1)	

	info = {}
	info.text = "View your procs"
	info.value = 8
	info.func = ShadowSeer_FrameDropDownMenu_ViewProcs
	UIDropDownMenu_AddButton(info, 1)	
	
	info = {}
	info.hasArrow = true -- creates submenu
	info.text = "Reporting mode " .. ShadowSeer_ReportMode
	info.value = 9
	UIDropDownMenu_AddButton(info, 1)	
	
--	info = {}
--	info.text = "View resists"
--	info.value = 6
--	info.func = ShadowSeer_FrameDropDownMenu_ViewOwnResists
--	UIDropDownMenu_AddButton(info, 1)	
end

function ShadowSeer_FrameDropDown_InitSubMenu()
	local info = {}
	
	info.text = "Report in Say"
	info.value = "SAY"
	info.func = ShadowSeer_FrameDropDownMenu_ChangeReportMode
	UIDropDownMenu_AddButton(info, 2)

	local info = {}
	info.text = "Report to Party"
	info.value = "PARTY"
	info.func = ShadowSeer_FrameDropDownMenu_ChangeReportMode
	UIDropDownMenu_AddButton(info, 2)
	
	local info = {}
	info.text = "Report to Raid"
	info.value = "RAID"
	info.func = ShadowSeer_FrameDropDownMenu_ChangeReportMode
	UIDropDownMenu_AddButton(info, 2)

	local info = {}
	info.text = "Report to Guild"
	info.value = "GUILD"
	info.func = ShadowSeer_FrameDropDownMenu_ChangeReportMode
	UIDropDownMenu_AddButton(info, 2)

	local info = {}
	info.text = "Report in whisper"
	info.value = "WHISPER"
	info.func = ShadowSeer_FrameDropDownMenu_ChangeReportMode
	UIDropDownMenu_AddButton(info, 2)
end

function ShadowSeer_FrameDropDownMenu_ChangeReportMode()
	ShadowSeer_ReportMode = this.value
	ShadowSeer_ShowMessage("Reporting will now be done in the " .. this.value .. " channel.")
	ToggleDropDownMenu(1, nil, ShadowSeerFrameDropDown, this:GetName(), 55, 10)
end

function ShadowSeer_FrameDropDownMenu_Clear()
	ShadowSeer_Command("clear")
end

function ShadowSeer_FrameDropDownMenu_Hide()
	ShadowSeer_Command("hide")
end

function ShadowSeer_FrameDropDownMenu_Disable()
	ShadowSeer_Command("off")
end

function ShadowSeer_FrameDropDownMenu_Enable()
	ShadowSeer_Command("off")
end

function ShadowSeer_FrameDropDownMenu_Toggle()
	ShadowSeer_Command("toggle")
end

function ShadowSeer_FrameDropDownMenu_MarkISB()
	if (ShadowSeer_Selected_Index == -1) then
		return
	end
	local name = ShadowSeer_GetActiveList()[ShadowSeer_Selected_Index].name
	if (ShadowSeer_WarlockHasISB(name)) then
		ShadowSeer_MarkAsWithoutISB(name)
	else
		ShadowSeer_MarkAsWithISB(name)
	end
end

function ShadowSeer_FrameDropDownMenu_ViewRaid()
	ShadowSeer_ViewMode = "PlayerList"
	ShadowSeer_RedoBars()
end

function ShadowSeer_FrameDropDownMenu_ViewOwnDamage()
	ShadowSeer_ViewMode = "OwnSpells"
	ShadowSeer_RedoBars()
end

function ShadowSeer_FrameDropDownMenu_ViewProcs()
	ShadowSeer_ViewMode = "Trinkets"
	ShadowSeer_RedoBars()
end

function ShadowSeer_FrameDropDownMenu_ViewOwnResists()
	ShadowSeer_ViewMode = "OwnResists"
	ShadowSeer_RedoBars()
end

--
-- color
--
-- quick hack to color spells individually
function ShadowSeer_GetColorForSpell(spellname)
	name = string.lower(spellname)

	if (string.find(name, "curse")) then
		return 0.8, 0.2, 0.2
	elseif ((name == "life tap") or (name == "dark pact")) then
		return 0.2, 0.8, 0.2
	else
		return ShadowSeer_GetColorForName(name)
	end
end

function ShadowSeer_GetColorForName(name)
	local n = string.lower(name)
	if (string.find(n, "curse of") == 1) then
		n = string.sub(n, 9)
	elseif (string.find(n, "shadow ") == 1) then
		n = string.sub(n, 7)
	elseif (string.find(n, "shadow") == 1) then
		n = string.sub(n, 6)
	end

	local a = string.byte("a", 1)
	local r = string.byte(n, 1) -- quick hack to get a name colored
	local g = string.byte(n, 2)
	local b = string.byte(n, 3)

	r = (r - a) / 25
	g = (g - a) / 25
	b = (b - a) / 25
	
	local cmax = max(r,g,b) -- don't allow too dark values
	if (cmax < 0.33) then
		if (r == cmax) then
			r = 1 - r
		elseif (g == cmax) then
			g = 1 - g
		else
			b = 1 - b
		end
	end

	return r, g, b
end

--
-- reporting
--
function ShadowSeer_AddLineTo(msg, line, r, g, b)
	local linerec = {}
	linerec["text"] = line
	linerec["r"] = r
	linerec["g"] = g
	linerec["b"] = b
	table.insert(msg, linerec)
end

SHADOWSEER_SELECT_R = 0.8
SHADOWSEER_SELECT_G = 0.8
SHADOWSEER_SELECT_B = 1

function ShadowSeer_SetTooltipText(msg)
	if (not msg[1]) then
		return
	end
	if (not ShadowSeer_Tooltip_Index) then
		ShadowSeer_Tooltip_Index = 1
	elseif (ShadowSeer_Tooltip_Index > table.getn(msg)) then
		ShadowSeer_Tooltip_Index = 1
	end

	ShadowSeer_Tooltip:SetOwner(ShadowSeerFrame)

	if (ShadowSeer_Tooltip_Index == 1) then
		ShadowSeer_Tooltip:SetText(msg[1].text, SHADOWSEER_SELECT_R, SHADOWSEER_SELECT_G, SHADOWSEER_SELECT_B)
	else
		ShadowSeer_Tooltip:SetText(msg[1].text, msg[1].r, msg[1].g, msg[1].b)
	end
	
	for i = 2,table.getn(msg) do
		if (i == ShadowSeer_Tooltip_Index) then
			ShadowSeer_Tooltip:AddLine(msg[i].text, SHADOWSEER_SELECT_R, SHADOWSEER_SELECT_G, SHADOWSEER_SELECT_B)
		else
			ShadowSeer_Tooltip:AddLine(msg[i].text, msg[i].r, msg[i].g, msg[i].b)
		end
	end
	
	ShadowSeer_Tooltip:AddLine("", 0.5, 0.5, 0.5)
	ShadowSeer_Tooltip:AddLine("To report this data, use the hotkeys in bindings.", 0.5, 0.5, 0.5)

	ShadowSeer_Tooltip:Show()
	ShadowSeer_Tooltip_Visible = true
	ShadowSeer_Tooltip_Text = msg
end

function ShadowSeer_ReportTo(mode, msg)
	ShadowSeer_Report_ShowLine(mode)
end

function ShadowSeer_Report_Scroll_Up()
	if (not ShadowSeer_Tooltip_Text or not ShadowSeer_Tooltip_Visible) then
		return
	end

	local dest_reached = false
	repeat 
		ShadowSeer_Tooltip_Index = ShadowSeer_Tooltip_Index - 1
		if (ShadowSeer_Tooltip_Index == 0) then
			ShadowSeer_Tooltip_Index = 1
			dest_reached = true
		elseif (ShadowSeer_Tooltip_Text[ShadowSeer_Tooltip_Index].text == "" or ShadowSeer_Tooltip_Text[ShadowSeer_Tooltip_Index].text == " ") then
			dest_reached = false
		else
			dest_reached = true
		end
	until (dest_reached)

	ShadowSeer_SetTooltipText(ShadowSeer_Tooltip_Text)
end

function ShadowSeer_Report_Scroll_Down()
	if (not ShadowSeer_Tooltip_Text or not ShadowSeer_Tooltip_Visible) then
		return
	end

	local dest_reached = false
	repeat 
		ShadowSeer_Tooltip_Index = ShadowSeer_Tooltip_Index + 1
		if (ShadowSeer_Tooltip_Index == table.getn(ShadowSeer_Tooltip_Text) + 1) then
			ShadowSeer_Tooltip_Index = table.getn(ShadowSeer_Tooltip_Text)
			dest_reached = true
		elseif (ShadowSeer_Tooltip_Text[ShadowSeer_Tooltip_Index].text == "" or ShadowSeer_Tooltip_Text[ShadowSeer_Tooltip_Index].text == " ") then
			dest_reached = false
		else
			dest_reached = true
		end
	until (dest_reached)

	ShadowSeer_SetTooltipText(ShadowSeer_Tooltip_Text)
end

function ShadowSeer_Report_ShowLine(mode)
	if (not ShadowSeer_Tooltip_Text or not ShadowSeer_Tooltip_Visible) then
		return
	end
	
	if (ShadowSeer_ReportMode == "WHISPER") then
		if (ShadowSeer_WhisperTarget) then
			SendChatMessage(ShadowSeer_Tooltip_Text[ShadowSeer_Tooltip_Index].text, "WHISPER", nil, ShadowSeer_WhisperTarget)
		else
			ShadowSeer_ShowMessage("You haven't set a whisper target. Set one by typing /ss whisper name.")
		end
	else
		SendChatMessage(ShadowSeer_Tooltip_Text[ShadowSeer_Tooltip_Index].text, ShadowSeer_ReportMode)
	end
end
